import { Server } from "socket.io";
import { TSocket } from "../../utilities/Types";
import { writeFile } from "node:fs/promises";
import path from "path";
import fs from "fs/promises";

interface ITileData {
    [key: string]: Buffer;
}

/**
 * Handle game master upload tile event
 * @param socket
 * @param io
 */
export default function (socket: TSocket, io: Server) {
    socket.on('gm:tile:upload', async (data: ITileData, callback: (response: boolean) => void) => {
        try {
            if (socket.character?.role !== 'gm') {
                callback(false);
                return;
            }

            const public_folder = path.join(process.cwd(), 'public', 'tiles');

            // Ensure the folder exists
            await fs.mkdir(public_folder, { recursive: true });

            const files = await fs.readdir(public_folder);
            let highestNumber = -1;

            // Find the highest number in existing filenames
            for (const file of files) {
                const match = file.match(/^(\d+)\.png$/);
                if (match) {
                    const number = parseInt(match[1], 10);
                    if (number > highestNumber) {
                        highestNumber = number;
                    }
                }
            }

            const uploadPromises = Object.entries(data).map(async ([key, tileData], index) => {
                const filename = `${highestNumber + index + 1}.png`;
                const finalFilePath = path.join(public_folder, filename);
                await writeFile(finalFilePath, tileData);
            });

            await Promise.all(uploadPromises);

            callback(true);
        } catch (error) {
            console.error('Error uploading tiles:', error);
            callback(false);
        }
    });
}